/*****************************************************************************
 * bonjour.h
 *****************************************************************************
 * Copyright (C) 2005 the VideoLAN team
 * $Id: fe63c0896f1d8f922d67bbbbb03912d1be0d6ff6 $
 *
 * Authors: Jon Lech Johansen <jon@nanocrew.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

void *bonjour_start_service( vlc_object_t *, const char*, const char *, int,
                             char * );
void bonjour_stop_service( void * );
